/*
 * Exemple of using Bmp2Tile
 *
 * A Plan Adr = c000
 * window adr = b000
 * B Plan adr = e000
 * sprite adr = be00
 * HScrol adr = b800
 *
 * Rev1.0       18Oct99
 */

#include "genesis.h"

/* in TMAP.S  (Tiles for map) */
extern ulong pal_tmap[];
extern ulong Empty_Tile[];
extern uint  tmap0_HSize;
extern uint  tmap0_VSize;
extern ulong tmap0[];
extern uint  tmap1_HSize;
extern uint  tmap1_VSize;
extern ulong tmap1[];
extern uint  tmap2_HSize;
extern uint  tmap2_VSize;
extern ulong tmap2[];      
               
/* in TSPR.S  (Tiles for sprites) */
extern ulong pal_tspr[];
extern ulong tspr0[];
extern ulong tspr1[];


#define     ADR_TMAP0   32
#define     ADR_TMAP1   (ADR_TMAP0 + 32 * tmap0_HSize * tmap0_VSize)
#define     ADR_TMAP2   (ADR_TMAP1 + 32 * tmap1_HSize * tmap1_VSize)
#define     ADR_TSPR0   (ADR_TMAP2 + 32 * tmap2_HSize * tmap2_VSize)
#define     ADR_TSPR1   (ADR_TSPR0 + 32 * 4 * 4)



/***** MODIFY THE WIDTH ****/
/***** see INIT_GFX     ****/
uint  extWIDTH;


void Init_GFX();

/*************  Main program  ***********/
void main()
{

    register uint posx, posy, joy;
    register uint x, y;

    /* Initialize Genesis GFX registers */
    Init_GFX();

    /* Load in vram */
    dma_vram_copy(Empty_Tile, 0, 32);
    dma_vram_copy(tmap0, ADR_TMAP0, 32 * tmap0_HSize * tmap0_VSize);
    dma_vram_copy(tmap1, ADR_TMAP1, 32 * tmap1_HSize * tmap1_VSize);
    dma_vram_copy(tmap2, ADR_TMAP2, 32 * tmap2_HSize * tmap2_VSize);


    for (y=0; y<tmap0_VSize; y++)
        show_tiles(ADR_TMAP0/32 + y*tmap0_HSize, ADR_TMAP0/32 + y*tmap0_HSize + tmap0_HSize - 1, 0,0,y, APLAN, 0,0,0);

    for (y=0; y<tmap0_VSize; y++)
        show_tiles(ADR_TMAP1/32 + y*tmap1_HSize, ADR_TMAP1/32 + y*tmap1_HSize + tmap1_HSize - 1, 0,0,y + tmap0_VSize, APLAN, 0,0,0);

    for (y=0; y<tmap0_VSize; y++)
        show_tiles(ADR_TMAP2/32 + y*tmap2_HSize, ADR_TMAP2/32 + y*tmap2_HSize + tmap2_HSize - 1, 0,0,y + tmap0_VSize + tmap1_VSize, APLAN, 0,0,0);

    dma_vram_copy(tspr0, ADR_TSPR0, 32*4*4);
    dma_vram_copy(tspr1, ADR_TSPR1, 32*1*4);

    posx = 10;
    posy = 20;
                                /* A000 'coze pal 1 */
    def_sprite(1,posx,posy,0xf00 ,0xA000 + ADR_TSPR0/32);
    def_sprite(2,posx,posy+31,0xc00,0xA000 + ADR_TSPR1/32);
    show_sprite(1,2);

    set_colors(0, pal_tmap);
    set_colors(1, pal_tspr);

    init_joypad();

    while(1)
    {
        joy= read_joypad1();

        if (joy)
        {
                switch(joy & 0xff)
                {
                        case JOY_RIGHT:
                                posx++;                              
                                break;
                        case JOY_LEFT:
                                posx--;
                                break;
                        case JOY_UP:
                                posy--;
                                break;
                        case JOY_DOWN:
                                posy++;
                                break;
                }
                def_sprite(1,posx,posy,0xf00 ,0xA000 + ADR_TSPR0/32);
                def_sprite(2,posx,posy+31,0xc00,0xA000 + ADR_TSPR1/32);
                show_sprite(1,2);
                wait_sync();
                
        }
    }                               
};

/*************  Initialize GFX  ***********/
void Init_GFX()
{
    register uint *pw;

    pw = (uint *) GFXCNTL;

    *pw = 0x8016;   /* reg. 0 - Enable HBL */
    *pw = 0x8174;   /* reg. 1 - Enable display, VBL, DMA + VCell size */
    *pw = 0x8230;   /* reg. 2 - Plane A =$30*$400=$C000 */
    *pw = 0x832C;   /* reg. 3 - Window  =$2C*$400=$B000 */
    *pw = 0x8407;   /* reg. 4 - Plane B =$7*$2000=$E000 */
    *pw = 0x855F;   /* reg. 5 - sprite table begins at $BE00=$5F*$200 */
    *pw = 0x8600;   /* reg. 6 - not used */
    *pw = 0x8702;   /* reg. 7 - Background Color number*/
    *pw = 0x8800;   /* reg. 8 - not used */
    *pw = 0x8900;   /* reg. 9 - not used */
    *pw = 0x8a01;   /* reg 10 - HInterrupt timing */
    *pw = 0x8b00;   /* reg 11 - $0000abcd a=extr.int b=vscr cd=hscr */
    *pw = 0x8c08;   /* reg 12 - hcell mode + shadow/highight + interlaced mode */
    *pw = 0x8d2E;   /* reg 13 - HScoll Table = $B800 */
    *pw = 0x8e00;   /* reg 14 - not used */
    *pw = 0x8f02;   /* reg 15 - auto increment data */
    *pw = 0x9000;   /* reg 16 - scrl screen v&h size */
    
    extWIDTH = 32;
    
    *pw = 0x9100;   /* reg 17 - window hpos */
    *pw = 0x92ff;   /* reg 18 - window vpos */
};
