/******************************************

 BMP2TILE 2.0
 by Kaneda

 Convert a BMP 16colors to Genesis ASM src
 to be included in Sozobon.
 Map Mode, tiles numbered as
     1 2 3 4
     5 6 7 8

 Sprite Mode (maxi 32x32), tiles numbered as
     1 3 5 7
     2 4 6 8

 Exit code
 0    Ok
 1    Bad Bmp
 2    Sprite Mode Size error
 3    Bad usage / Exe only launched
 5    Unknow option


 History
 10Aug98        BMP2TILE can convert to Map mode
 01Jan99        BMP2TILE compiled with GCC (no longer TC)
 15Jan99        GCC Getch() error...
 18Jan99        Sprite mode added	1.0 released
 13Oct99        Beginning of v2.0
		Palette also save in ASM
		Recompiled with TC
 14Oct99	Add selection of your sprite in Sprite Mode
		Fixed Height bug
 18Oct99	Fixed pal bug
		Add comment
		Released on my web page

 *******************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include "bmp.h"
#include "kgraph.h"
#include "interac.h"
#include <dos.h>


int AccessBMP (char *);                  // func to check BMP file
void ProcessBMP (char *);

RGBQUAD		BMPRGBQuad;
BITMAPFILEHEADER BMPHeader;
BITMAPINFOHEADER BMPInfo;

uint FirstTile;
uint SpriteOrMap;            	//if True, Sprite
uint MaxHSize;                  //Max Tile HSize (32 or BMP Hsize)
uint MaxVSize;

int main (int argc, char *argv[])
{
	int x;
	int argn = 1;

	argc--;

	FirstTile = 0;              //if 1, create the first empty tile
	SpriteOrMap = 0;	    //if 1, Sprite mode


	if (argc == 0)
	{
		printf("\n\aUsage: BMP2Tile [-f] [-s] <filename.BMP>\n");
		printf("\t-f  create a first empty tile.\n");
		printf("\t-s  create tiles for Sprite use.\n\n");
		printf("Bmp2Tile 2.0 by Kaneda\nGenesis rulez!!\n");
		exit(3);
	}



	while (argv[argn][0] == '-' && argc)
	{
		switch (argv[argn][1])
		{
		    case 'f':
				FirstTile = 1;
				break;
		    case 's':
				SpriteOrMap = 1;
				break;
		    default:
				printf ("*ERROR*\t :\n\tUnknown option '%c'\n", argv[argn][1]);
				exit (5);
				break;
		}
		argn++;
		argc--;
	}

	x  = AccessBMP(argv[argn]);                // see below


	clrscr();

	// new in v2.0
	/*
	if (SpriteOrMap && (BMPInfo.biWidth > 32 || BMPInfo.biHeight>32 ) )
	{
		printf("In Sprite Mode, size is 32x32 max!");
		exit(2);
	}
	*/

	//print BMP info
	printf("\nFILE: %s",argv[argn]);
	printf("\n");
	printf("\nType de BMP: ");

	if (BMPInfo.biBitCount == 1)
		printf("Black & White");
	else if (BMPInfo.biBitCount == 4)
		printf("16 Colors");
	else if (BMPInfo.biBitCount == 8)
		printf("256 Colors");
	else
		printf("24-Bit RGB");

	if (BMPInfo.biCompression==0)
		printf("\nCompression : None");
	else
		printf("\nCompression : %u",BMPInfo.biCompression);

	printf("\nStructure size: %u",BMPHeader.bfSize-BMPHeader.bfOffBits);
	printf("\nHorizontal res. : %u",BMPInfo.biXPelsPerMeter);
	printf("\nVertical res. : %u",BMPInfo.biYPelsPerMeter);
	printf("\nWidth : %u",BMPInfo.biWidth);
	printf("\nHeight : %u",BMPInfo.biHeight);
	printf("\nBits per pixel: %u",BMPInfo.biBitCount);
	printf("\n");

	switch(x)
	{
		case BMP_OK:            printf("\n\tProceeding to Convert");
					break;
		case BMP_TooBig:        printf("\n\tBMP too big...");
					break;
		case BMP_WrongVideo:    printf("\n\tBMP must be in 16 colors.");
					break;
		case BMP_Compression:   printf("\n\tBMP Compressed.");
					break;
		default:                printf("\n\t* OOOPS *");
					printf("\n\t* ABORT *");
					break;
	}

	if (x==BMP_OK)
	{
		if (SpriteOrMap)
		   printf(" in Sprite Mode");
		else
		   printf(" in Map Mode");

		printf("\n\tPress any Key...\n");
		/* must put a \n else you won't see it...Getch() bug? */
		getch();

		ProcessBMP(argv[argn]); 	//see below

		printf("All done...\n");
		printf("You must have <bmp_name>.S containing Tile and Pal.\n");
		printf("\tHappy Dev!");

		return(0);
	}
	else
		return (1);
};


//////////////////////////////////////////////////////////////////////////
// AccessBMP(char *Filename)
//             This routine opens the BMP file and checks to see if it's
//             okay to use, i.e 16-Colors & Uncompressed.
//             BMP file structures should be defined as global
//             in the calling routine.
//////////////////////////////////////////////////////////////////////////
int AccessBMP(char *BMPFile)
{
	FILE *Fn;
	int errorcode, maxx, maxy;


	Fn = fopen(BMPFile,"rb");               // Open up BMP image file

						// Read in BMP header
	fread(&BMPHeader.bfType    ,sizeof(int)    	 ,1,Fn);
	fread(&BMPHeader.bfSize    ,sizeof(unsigned long),1,Fn);
	fread(&BMPHeader.bfRes     ,sizeof(int)          ,1,Fn);
	fread(&BMPHeader.bfRes2    ,sizeof(int)          ,1,Fn);
	fread(&BMPHeader.bfOffBits ,sizeof(unsigned long),1,Fn);
	printf("%d", BMPHeader.bfOffBits);

	fread(&BMPInfo,sizeof(BMPInfo),1,Fn);
	fclose(Fn);

	errorcode=BMP_OK;

						// Error: only 16-color BMPs can be processed
	if (BMPInfo.biBitCount != 4)
		errorcode = BMP_WrongVideo;

						// Error: image was compressed
	if (BMPInfo.biCompression != 0)
		errorcode = BMP_Compression;

	return(errorcode);
};

//////////////////////////////////////////////////////////////////////////
// ProcessBMP(char *Filename)
//             This routine opens the BMP file, displays it pixel by
//             pixel and let the user do his business.
//             Call AccessBMP before using this routine.  BMP file
//             structures should be defined as global vars in the calling
//             routine.
//////////////////////////////////////////////////////////////////////////
void ProcessBMP(char *BMPFile)
{
	FILE *Fn, *Fs;
	uint i, x, y;
	uchar Color, VideoMode;
	char buff[12];
	uchar i_r, i_v, i_b;

	Fn = fopen(BMPFile,"rb");

	strtok(BMPFile,".");                    // "BMP_File_Name.S"
	strcpy(buff, BMPFile);
	strcat(buff,".s");

	Fs = fopen(buff,"wb");



						// Jump to image data in
	fseek(Fn,BMPHeader.bfOffBits,SEEK_SET); // BMP file

    /*
	if (BMPInfo.biWidth <= 640 & BMPInfo.biHeight <= 480)	VideoMode=0x12;
	if (BMPInfo.biWidth <= 640 & BMPInfo.biHeight <= 200)	VideoMode=0x0E;
	if (BMPInfo.biWidth <= 320 & BMPInfo.biHeight <= 200)	VideoMode=0x0D;
    */

    KSetVideoMode(0x13);

	// Define the Selector limits (see Interact.c)
	if (SpriteOrMap)
	{
		MaxHSize=min(BMPInfo.biWidth/8,4);
		MaxVSize=min(BMPInfo.biHeight/8,4);
		if (BMPInfo.biWidth%8 >0) MaxHSize++;    //if Width=26, MaxHSize MUST be 32
		if (BMPInfo.biHeight%8>0) MaxVSize++;
		MaxHSize*=8;
		MaxVSize*=8;
	}
	else
	{
		MaxHSize=BMPInfo.biWidth;
		MaxVSize=BMPInfo.biHeight;
	}

	fseek(Fn,54L, SEEK_SET);
/*

   for (y=0; y<16; y++)
   {
        for (x=0; x<320; x++)
        {
            KWritePix(0, y, x, y);
        }
   }

   getch();
*/
	// make the pal
	fprintf(Fs,"\t.globl _pal_%s\n\n_pal_%s:\n", BMPFile, BMPFile);

	for (i=0;i<16;i++)
	{
		fread(&BMPRGBQuad,sizeof(BMPRGBQuad),1,Fn);
        i_b = BMPRGBQuad.rgbBlue;
		i_v = BMPRGBQuad.rgbGreen;
		i_r = BMPRGBQuad.rgbRed;

        KSetColor(i, i_r>>2, i_v>>2, i_b>>2);

		fprintf(Fs,"\tdc.w\t$%0.1x%0.1x%0.1x\n", i_b >> 4 , i_v >> 4 , i_r >> 4);
	}

	fprintf(Fs,"\n");

	fclose(Fs);

						// Read in & display BMP
						// Note: BMP's are stored in
						// bottom-up fashion, so we'll
						// draw from the bottom to the top
						// Also, since the program is setup
						// only for 16-Color BMP's, there
						// are 4 bits for each pixel
    
  /*
   for (y=0; y<16; y++)
   {
        for (x=0; x<320; x++)
        {
            KWritePix(0, y, x, y+20);
        }
   }
            
   */



	y = BMPInfo.biHeight-1;

	do
	{
		for (x=0; x < BMPInfo.biWidth; x++)
		{
			fread(&Color,sizeof(Color),1,Fn);
            KWritePix(0, Color>>4, x++, y);
            KWritePix(0, Color&0xf, x, y);
		}

	} while (y-- > 0);

    fclose(Fn);

	// pal read, BMP drawn, let's do the convertion
	Interact(BMPFile);

	KSetVideoMode(0x03);
};
